# Return vector of stick given its angular definition
def polar_to_cartesian(az, el, length = 1):
    x = length * float(sin(az)*cos(el))
    y = length * float(cos(az)*cos(el))
    z = length * float(sin(el))
    return [x,y,z]
   
# Takes an angle in degrees and returns a float representing radians
def r_from_d(angle):
    return float(angle/360*2*pi)

# Returns vector of the stick given polar definition
def stick_vector(az, el, length, base_vector = vector([0,0,0])):
    return base_vector + vector(polar_to_cartesian(az, el, length))

# Returns vector of sun given azimuth and elevation
def sun_vector(az, el):
    return vector(polar_to_cartesian(az, el))

# Takes four sage Vector objects. The 'normal' vector is normal to the
# plane that one would like to cast a shadow on. The 'anchor' vector
# is the initial point of the normal vector with respect to the horizon
# coordinate system. The 'sun' vector is one that points to the sun from
# origin of the horizon plane. The 'stick' vector is the vector pointing
# from the horizon origin to the tip of the stick. 
def get_shadow_vector(normal, anchor, sun, stick):
    
    sun_and_stick = sun + stick
    # Numerator variable
    num = normal.dot_product(anchor - stick)
    # Denominator variable
    den = normal.dot_product(sun_and_stick - stick)
    u = num / den
    return stick + u * (sun_and_stick - stick)

# Takes normal and anchor vectors to define shadow plane, as well as the
# sun vector pointing to the sun to cast projections from both endpoints
# of our shadow casting stick defined by the vectors stick_start and
# stick_end
def get_stick_endpoint_shadows(normal, anchor, sun, stick_start, stick_end):
    # Check to see if the Sun is below the horizon and return null if
    # it is because there would be no shadows
    if sun[2] <= 0:
        return None
    shadow_start = get_shadow_vector(normal, anchor, sun, stick_start)
    shadow_end = get_shadow_vector(normal, anchor, sun, stick_end)
    return (shadow_start, shadow_end)

